///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
 *	OPCODE - Optimized Collision Detection
 *	Copyright (C) 2001 Pierre Terdiman
 *	Homepage: http://www.codercorner.com/Opcode.htm
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Contains code to perform "picking".
 *	\file		OPC_Picking.h
 *	\author		Pierre Terdiman
 *	\date		March, 20, 2001
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 * Updated by Stephen G. Tuggy 2021-07-03
 * Updated by Stephen G. Tuggy 2022-01-06
 * Updated by Benjamen R. Meyer 2023-05-27
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef VEGA_STRIKE_ENGINE_CMD_COLLSION2_ICE_OPC_PICKING_H
#define VEGA_STRIKE_ENGINE_CMD_COLLSION2_ICE_OPC_PICKING_H

#ifdef OPC_RAYHIT_CALLBACK

enum CullMode {
    CULLMODE_NONE = 0,
    CULLMODE_CW = 1,
    CULLMODE_CCW = 2
};

typedef CullMode (*CullModeCallback)(uint32_t triangle_index, void *user_data);

OPCODE_API bool SetupAllHits(RayCollider &collider, CollisionFaces &contacts);
OPCODE_API bool SetupClosestHit(RayCollider &collider, CollisionFace &closest_contact);
OPCODE_API bool SetupShadowFeeler(RayCollider &collider);
OPCODE_API bool SetupInOutTest(RayCollider &collider);

OPCODE_API bool Picking(
        CollisionFace &picked_face,
        const Ray &world_ray, const Model &model, const Matrix4x4 *world,
        float min_dist, float max_dist, const Point &view_point, CullModeCallback callback, void *user_data);
#endif

#endif //VEGA_STRIKE_ENGINE_CMD_COLLSION2_ICE_OPC_PICKING_H
